#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SNMP-Check for Aruba CX Switches (PSUs, Temperatures, Fans)"""

# +------------------------------------------------------------+
# |                                                            |
# |             | |             | |            | |             |
# |          ___| |__   ___  ___| | ___ __ ___ | | __          |
# |         / __| '_ \ / _ \/ __| |/ / '_ ` _ \| |/ /          |
# |        | (__| | | |  __/ (__|   <| | | | | |   <           |
# |         \___|_| |_|\___|\___|_|\_\_| |_| |_|_|\_\          |
# |                                   custom code by SVA       |
# |                                                            |
# +------------------------------------------------------------+
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright (C) 2023  SVA System Vertrieb Alexander GmbH
#                       by leon.buhleier@sva.de

# resides under ~/local/lib/python3/cmk/base/plugins/agent_based

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    State,
    Result,
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
)

from cmk.base.plugins.agent_based.utils.temperature import (
    check_temperature,
)

from cmk.base.plugins.agent_based.utils.fan import (
    check_fan
)


# PSU plugin

def discover_aruba_cx_psu(section) -> DiscoveryResult:
    # print("Discovery-PSU", section)
    for key in section["psus"]:
        yield Service(
            item = key,
        )

def parse_psu_summary(summary_raw) -> str:
    parsed = "[" + summary_raw["state"] + "], "
    parsed += "Number of Failures: " + summary_raw["number_failures"] + ", "
    parsed += "Maximum Power: " + summary_raw["maximum_power"] + " W, "
    parsed += "Product Name: " + summary_raw["product_name"] + ", "
    parsed += "Serial Number: " + summary_raw["serial_number"]
    return parsed


def check_aruba_cx_psu(item, section) -> CheckResult:
    # print("Check-PSU", item, section)
    if item in section["psus"]:
        service_state = State.CRIT
        if section["psus"][item]["state"] == "ok":
            service_state = State.OK
        service_summary = parse_psu_summary(section["psus"][item])
        service_details = service_summary.replace(", ", "\n")
        yield Result(
            state = service_state,
            summary = service_summary,
            details = service_details,
        )

register.check_plugin(
    name = "aruba_cx_psu",
    sections = ["aruba_cx_parsed"],
    service_name = "PSU %s",
    discovery_function = discover_aruba_cx_psu,
    check_function = check_aruba_cx_psu,
)

# Temperature Plugin

def discover_aruba_cx_temp(section) -> DiscoveryResult:
    # print("Discovery-Temp", section)
    for key in section["temperatures"]:
        yield Service(
            item = key,
        )

def check_aruba_cx_temp(item, params, section) -> CheckResult:
    # print("Check-Temp", item, params, section)
    if item in section["temperatures"]:
        service_reading = section["temperatures"][item]["temperature"]
        service_dev_unit = "c"
        service_dev_status = State.CRIT
        if section["temperatures"][item]["state"] == "normal":
            service_dev_status = State.OK
        yield from check_temperature(
            reading = service_reading,
            params = params,
            dev_unit = service_dev_unit,
            dev_status = service_dev_status,
        )

register.check_plugin(
    name = "aruba_cx_temp",
    sections = ["aruba_cx_parsed"],
    service_name = "Temperature %s",
    discovery_function = discover_aruba_cx_temp,
    check_function = check_aruba_cx_temp,
    check_ruleset_name = "temperature",
    check_default_parameters = {},
)

# Fan Plugin

def discover_aruba_cx_fan(section) -> DiscoveryResult:
    # print("Discovery-Fan", section)
    for key in section["fans"]:
        yield Service(
            item = key,
        )

def check_aruba_cx_fan(item, params, section) -> CheckResult:
    # print("Check-Fan", item, params, section)
    if item in section["fans"]:
        service_rpm = int(section["fans"][item]["rpm"])
        if section["fans"][item]["state"] != "ok":
            yield Result(
                state = State.CRIT,
                summary = "Fanstate not OK",
            )
        yield from check_fan(
            rpm = service_rpm,
            params = params,
        )

register.check_plugin(
    name = "aruba_cx_fan",
    sections = ["aruba_cx_parsed"],
    service_name = "Fan %s",
    discovery_function = discover_aruba_cx_fan,
    check_function = check_aruba_cx_fan,
    check_ruleset_name = "hw_fans",
    check_default_parameters = {},
)