#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""SNMP-section for Aruba CX Switches (PSUs, Temperatures, Fans)"""

# +------------------------------------------------------------+
# |                                                            |
# |             | |             | |            | |             |
# |          ___| |__   ___  ___| | ___ __ ___ | | __          |
# |         / __| '_ \ / _ \/ __| |/ / '_ ` _ \| |/ /          |
# |        | (__| | | |  __/ (__|   <| | | | | |   <           |
# |         \___|_| |_|\___|\___|_|\_\_| |_| |_|_|\_\          |
# |                                   custom code by SVA       |
# |                                                            |
# +------------------------------------------------------------+
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   Copyright (C) 2023  SVA System Vertrieb Alexander GmbH
#                       by leon.buhleier@sva.de

# resides under ~/local/lib/python3/cmk/base/plugins/agent_based

from typing import (
    List,
    Dict,
)

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    startswith,
    all_of,
    exists,
    any_of
)

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    StringTable,
)

from cmk.base.api.agent_based.section_classes import (
    SNMPTree,
)


def parse_aruba_cx(string_table: List[StringTable]) -> Dict:
    # print("Parse-Input", str(string_table))
    
    oid_tree = {"psus": 0, "temperatures": 1, "fans": 2}
    parsed = {}

    parsed["psus"] = {}
    for psu_name, psu_state, psu_product_name, psu_serial_number, psu_maximum_power, psu_number_failures in string_table[oid_tree["psus"]]:
        parsed["psus"][psu_name] = {
            "state": psu_state,
            "product_name": psu_product_name,
            "serial_number": psu_serial_number,
            "maximum_power": psu_maximum_power,
            "number_failures": psu_number_failures,
        }

    parsed["temperatures"] = {}
    for temp_sensor_name, temp_sensor_state, temp_sensor_temperature in string_table[oid_tree["temperatures"]]:
        temp_sensor_temperature_parsed = int(temp_sensor_temperature) / 1000
        parsed["temperatures"][temp_sensor_name] = {
            "state": temp_sensor_state,
            "temperature": temp_sensor_temperature_parsed,
        }
    
    parsed["fans"] = {}
    for fan_name, fan_state, fan_product_name, fan_serial_number, fan_rpm in string_table[oid_tree["fans"]]:
        parsed["fans"][fan_name] = {
            "state": fan_state,
            "product_name": fan_product_name,
            "serial_number": fan_serial_number,
            "rpm": fan_rpm,
        }

    # print("Parse-Output", str(parsed))
    return parsed

register.snmp_section(
    name = "aruba_cx",
    parse_function = parse_aruba_cx,
    parsed_section_name = "aruba_cx_parsed",
    detect = all_of(
        startswith(".1.3.6.1.2.1.1.2.0", ".1.3.6.1.4.1.47196.4.1.1.1"),
        any_of(
            exists(".1.3.6.1.4.1.47196.4.1.1.3.11.2.1.1.3.*"),
            exists(".1.3.6.1.4.1.47196.4.1.1.3.11.3.1.1.5.*"),
            exists(".1.3.6.1.4.1.47196.4.1.1.3.11.5.1.1.4.*"),
        ),
    ),
    fetch = [
        SNMPTree(
            base = ".1.3.6.1.4.1.47196.4.1.1.3.11.2.1.1", # PSU
            oids = [
                "3",    # PSUName
                "4",    # PSUState
                "5",    # PSUProductName
                "6",    # PSUSerialNumber
                "8",    # PSUMaximumPower
                "9",    # PSUNumberFailures
            ],
        ),
        SNMPTree(
            base = ".1.3.6.1.4.1.47196.4.1.1.3.11.3.1.1", # Temperature
            oids = [
                "5",    # TempSensorName
                "6",    # TempSensorState
                "7",    # TempSensorTemperature 
            ],
        ),
        SNMPTree(
            base = ".1.3.6.1.4.1.47196.4.1.1.3.11.5.1.1", # Fan
            oids = [
                "4",    # FanName
                "5",    # FanState
                "6",    # FanProductName
                "7",    # FanSerialNumber
                "8",    # FanRPM
            ],
        ),
    ],
)